import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Scanner;

public class GameLogToNarsese {

    private static int gameNumber = 1;
    private static String gameName = "{game" + gameNumber + "}";
    private static String roundName = "";

    private static int teamNumber = 1;
    private static int numPlayers = 5;
    private static int numSpies = 2;
    private static double spyPercent = 0.40;

    private static boolean spiesKnown = false;

    public static void main(String[] args){
        //Run either testing or training games
        String inputDirectory = "gameData/playedGameData/testingGames/";
        String outputDirectory = "gameData/narseseGameData/testingGames/";
        for(int j = 0; j <= 1; j++)
        {
            if (j == 1){
                inputDirectory = "gameData/playedGameData/trainingGames/";
                outputDirectory = "gameData/narseseGameData/trainingGames/";
                teamNumber = 1;
            }
            for(int i = 1; i <=10; i++)
            {
                gameNumber = i;
                gameName = "{game" + gameNumber + "}";
                String filename = "sampleGame" + gameNumber;
                roundName = "";

                Scanner sc;
                try {
                    sc = new Scanner(new File(inputDirectory +filename + ".txt"));
                } catch (FileNotFoundException e) {
                    throw new RuntimeException(e);
                }
                try (FileWriter fw = new FileWriter(outputDirectory + filename + ".nal")) {
                    fw.write(getRules());
                    fw.write("<" + gameName + " --> game>. %1.00;0.99%\n");
                    while(sc.hasNextLine()){
                        String line = sc.nextLine();
                        String translated = translate(line);
                        fw.write(translated);
                        if(!translated.isBlank())
                            fw.write("5\n"); //give 5 cycles to process
                    }
                    //fw.write(getQuestions());
                    sc.close();
                } catch (IOException e) {
                    throw new RuntimeException(e);
                }
            }
        }
    }

    public static String translate(String message) {
        if(message.isBlank())
            return "";

        //The spies are: C, E,
        //'C is a spy'
        //'E is a spy'
        //<C --> spy>.
        //<E --> spy>.
        if(message.contains("The spies are:"))
        {
            spiesKnown = true;
            String narsese = "";
            ArrayList<String> spies = new ArrayList<>();
            String allNames = message.split(":")[1];
            String[] names = allNames.split(",");
            for (String name : names) {
                if(!name.isBlank())
                    spies.add(name);
            }
            for (String spy : spies){
                narsese += "<{" + spy.trim() + gameNumber + "} --> spy>. %1.00;0.99%\n";
            }
            return narsese;
        }

        //The resistance are: A, B, D,
        if(message.contains("The resistance are:"))
        {
            String narsese = "";
            ArrayList<String> resistance = new ArrayList<>();
            String allNames = message.split(":")[1];
            String[] names = allNames.split(",");
            for (String name : names) {
                if(!name.isBlank())
                    resistance.add(name);
            }
            for (String r : resistance){
                narsese += "<{" + r.trim() + gameNumber + "} --> spy>. %0.00;0.99%\n";
            }
            return narsese;
        }
        //Number of players: 5
        if(message.contains("Number of players")){
            numPlayers = Integer.parseInt(message.substring(message.length()-1));
            String narsese = "";
            for(int i = 1; i <= numPlayers; i++)
                narsese += "<{" + Player.playerNames.get(i) + gameNumber + "} --> player>. %1.00;0.99%\n";
            return narsese;
        }
        //Number of spies: 2
        if(message.contains("Number of spies")){
            numSpies = Integer.parseInt(message.substring(message.length()-1));
            spyPercent = numSpies/(double) numPlayers;
            if(spiesKnown)
                return "";
            String narsese = "";
            for(int i = 1; i <= numPlayers; i++)
                narsese += "<{" + Player.playerNames.get(i) + gameNumber + "} --> spy>. :|: %" + spyPercent + ";0.60%\n";
            return narsese;
        }

        if(message.contains("Round") || message.contains("Failed") || message.contains("Leader"))
        {
            String[] input = message.split(":");
            if(input[0].contains("Leader")) {
                //Leader: A
                //<{A} --> leader>.
                if(message.contains(":"))
                    return "<{" + input[1].trim() + gameNumber + "} --> leader>. :|: %1.00;0.9%\n";
                else{
                    //Leader passes to B
                    String newLeader = message.substring(message.length()-1).trim();
                    int newLeaderIndex = 0;
                    for(int i = 1; i <= numPlayers; i++)
                        if(Player.playerNames.get(i).equals(newLeader))
                        {
                            newLeaderIndex = i;
                            break;
                        }
                    int oldLeaderIndex = newLeaderIndex - 1;
                    if(oldLeaderIndex == 0)
                        oldLeaderIndex = numPlayers;
                    String oldLeader = Player.playerNames.get(oldLeaderIndex);
                    return "(--,<{" + oldLeader + gameNumber +"} --> leader>). :|: %1.00;0.9%\n"+
                            "<{" + newLeader + gameNumber + "} --> leader>. :|: %1.00;0.9%\n"
                            + getQuestions(); //ASK questions after each leader change
                }
            }
            //Round: 1
            //'{game1} is in {round1}'
            //<{round1} --> round>.
            //<(*,{game1},{round1}) --> in>.
            if(input[0].contains("Round")){
                String previousRound = roundName;
                roundName = "{round" + input[1].trim() + "g" + gameNumber + "}";
                if(previousRound.equals(roundName)) //round didn't change, just leadership change
                    return "";
                String output = "<" + roundName + " --> round>. %1.00;0.9%\n"
                        + "<(*," + gameName + "," + roundName + ") --> in>. :|: %1.00;0.9%\n";
                if(!previousRound.isBlank())
                    return output + "(--,<(*," + gameName + "," + previousRound + ") --> in>). :|: %1.00;0.9%\n";
                return output;
            }

            //Failed Team Votes: 0
            //<{round1} --> [<failedVotes --> [0]>]>.
            if(input[0].contains("Votes"))
                return "<" + roundName + " --> [<failedVotes --> [" + input[1].trim() +"]>]>. :|: %1.00;0.9%\n";

            //Mission Score: Succeeded = 0, Failed = 0
            //<{game1} --> [<resistanceScore --> [0]>]>.
            //<{game1} --> [<spyScore --> [0]>]>.
            String resistanceScore = input[1].substring(input[1].indexOf('=') + 2, input[1].indexOf('=') + 3);
            String spyScore = input[1].substring(input[1].lastIndexOf('=') + 2);
            return "<{game" + gameNumber + "} --> [<resistanceScore --> [" + resistanceScore + "]>]>. :|: %1.00;0.9%\n" +
                "<{game" + gameNumber + "} --> [<spyScore --> [" + spyScore + "]>]>. :|: %1.00;0.9%\n";

        }

        //Deliberation rules
        if(message.contains("Deliberation"))
        {
            //Deliberation phase begins.
            //<{game1} --> deliberating>.
            if(message.contains("begins")){
                return "<" + gameName + " --> deliberating>. :|: %1.00;0.9%\n";
            }
            //Deliberation phase ends.
            //(--,<{game1} --> deliberating>).
            return "<" + gameName + " --> deliberating>. :|: %0.00;0.9%\n";
        }

        if(message.contains("vouch") || message.contains("accuses") || message.contains("deliberating"))
        {
            String[] words = message.split(" ");
            String firstPlayer = words[0] + gameNumber;
            String action = words[1];
            if(action.equals("vouches")) {
                //A vouches for C. (A believes that C is not a spy)
                //<(*,{A},(--, <{C} --> spy>)) --> believes>.
                String secondPlayer = words[3].replace(".", "") + gameNumber;
                return "<(*, {" + firstPlayer + "}, (--, <{" + secondPlayer + "} --> spy>)) --> believes>. :|: %0.9;0.9%\n";
            }
            else if (action.equals("accuses")){
                //B accuses A of being a spy. (B believes that C is a spy)
                String secondPlayer = words[2] + gameNumber;
                return "<(*, {" + firstPlayer + "}, <{" + secondPlayer + "} --> spy>) --> believes>. :|: %0.9;0.9%\n";
            }
            else{
                //D is done deliberating.
                return "";
            }
        }

        if(message.contains("proposes") || message.contains("proposed"))
        {
            //A Proposes the team: [A, C]
            //A votes Y on proposed team [A, C]
            String teamName = "{team" + teamNumber + "g" + gameNumber + "}";
            String firstName = "{" + message.substring(0, 1) + gameNumber + "}";
            String team = message.substring(message.indexOf("[") + 1, message.indexOf("]"));
            List<String> teamMembers = Arrays.asList(team.split(",")); //parse team members into list

            if(message.contains("proposes"))
            {
                //A Proposes the team: [A, C]
                String narsese = "";
                //<{team1} --> team>.
                narsese += "<" + teamName + " --> team>. %1.00;0.99%\n";
                //<{team1} --> [<size --> [2]>]>.
                narsese += "<" + teamName + " --> [<size --> [" + teamMembers.size() + "]>]>. %1.00;0.9%\n";
                //<(*,{A},{team1}) --> proposes>.
                narsese += "<(*," + firstName + "," + teamName + ") --> proposes>. %1.00;0.9%\n";
                //<(*,{team1},{round1}) --> proposedOn>.
                narsese += "<(*," + teamName + "," + roundName + ") --> proposedOn>. %1.00;0.9%\n";
                //<(*,{A},{team1}) --> memberOf>.
                //<(*,{C},{team1}) --> memberOf>.
                for(String s : teamMembers)
                    narsese += "<(*,{" + s.trim() + gameNumber + "}," + teamName + ") --> memberOf>. %1.00;0.99%\n";
                teamNumber ++;
                return narsese;
            }
            else{
                //A votes Y on proposed team [A, C]
                if(message.split(" ")[2].equalsIgnoreCase("Y"))
                    return "<(*," + firstName + "," + teamName + ") --> supports>. :|: %0.9;0.8%\n";
                else
                    return "(--,<(*," + firstName + "," + teamName + ") --> supports>). :|: %0.9;0.8%\n";
            }
        }

        //Vote passes.
        if(message.equals("Vote passes."))
            return "<{team" + teamNumber + "g" + gameNumber + "} --> [accepted]>. %1.00;0.99%\n";

        //Vote fails.
        if(message.equals("Vote fails."))
            return "(--,<{team" + teamNumber + "g" + gameNumber + "} --> [accepted]>). %1.00;0.99%\n";

        //The mission passes with 0 failure votes.
        if(message.contains("The mission passes with 0"))
            return "<{team" + teamNumber + "g" + gameNumber + "} --> [success]>. %1.00;0.99%\n" + "<{team" + teamNumber + "g" + gameNumber + "} --> [noFails]>. %1.00;0.99%\n";

        //The mission passes with 1 failure votes.
        if(message.contains("The mission passes with 1"))
            return "<{team" + teamNumber + "g" + gameNumber + "} --> [success]>. %1.00;0.99%\n" + "<{team" + teamNumber + "g" + gameNumber + "} --> [oneFail]>. %1.00;0.99%\n";

        //The mission fails with 1 failure votes.
        if(message.contains("The mission fails with 1"))
            return "(--,<{team" + teamNumber + "g" + gameNumber + "} --> [success]>). %1.00;0.99%\n" + "<{team" + teamNumber + "g" + gameNumber + "} --> [oneFail]>. %1.00;0.99%\n";

        //The mission fails with 2 failure votes.
        if(message.contains("The mission fails with 2"))
            return "(--,<{team" + teamNumber + "g" + gameNumber + "} --> [success]>). %1.00;0.99%\n" + "<{team" + teamNumber + "g" + gameNumber + "} --> [twoFails]>. %1.00;0.99%\n";

        //3 failed missions, the spies win.
        //5 failed votes, the resistance couldn't come to a consensus. The spies win.
        if(message.contains("spies win."))
            return "<"+gameName + " --> [spyVictory]>. :|: %1.00;0.99%\n";
        //3 successful mission, the resistance win.
        else if(message.contains("resistance win."))
            return "(--,<"+gameName + " --> [spyVictory]>). :|: %1.00;0.99%\n";

        return "";
    }

    public static String getRules(){
        //RULES AND LOGIC
        String member = "<(*, $1, #2) --> memberOf>";
        String isPlayer = "<$1 --> player>";
        String isTeam = "<#2 --> team>";
        String successfulTeam = "<#2 --> [success]>";
        String failingTeam = "(--,<#2 --> [success]>)";
        String isMemberOfSuccessfulTeam = "(&&," + isPlayer + "," + isTeam + "," + successfulTeam + "," + member + ")";
        String isMemberOfFailingTeam = "(&&," + isPlayer + "," + isTeam + "," + failingTeam + "," + member + ")";

        //more likely to not be a spy if on a successful mission
        double resistancePercent = 1-(spyPercent/2);
        String strRestPercent = String.format("%f2", resistancePercent);
        String ruleMissionSuccess = "<" + isMemberOfSuccessfulTeam + " ==> (--,<$1 --> spy>)>. %" + strRestPercent + ";0.9%\n";

        //more likely to be a spy if on a failing mission
        String strSpyPercent = String.format("%f2", spyPercent*2);
        String ruleMissionFail = "<" + isMemberOfFailingTeam + " ==> <$1 --> spy>>. %" + strSpyPercent + ";0.9%\n";

        //TODO: If a mission fails then at least one player is a spy

        //Rules about proposing and voting for or against failing and successful teams
        String supports = "<(*,$1, #2) --> supports>";
        String against = "(--,<(*,$1, #2) --> supports>)";
        String proposes = "<(*,$1, #2) --> proposes>";
        String supportsSuccessfulTeam = "(&&," + isPlayer + "," + isTeam + "," + successfulTeam  + "," + supports + ")";
        String supportsUnsuccessfulTeam = "(&&," + isPlayer + "," + isTeam + "," + failingTeam  + "," + supports + ")";
        String againstSuccessfulTeam = "(&&," + isPlayer + "," + isTeam + "," + successfulTeam  + "," + against + ")";
        String againstUnsuccessfulTeam = "(&&," + isPlayer + "," + isTeam + "," + failingTeam   + "," + against + ")";
        String proposesSuccessfulTeam = "(&&," + isPlayer + "," + isTeam + "," + successfulTeam  + "," + proposes + ")";
        String proposesUnsuccesfulTeam = "(&&," + isPlayer + "," + isTeam + "," + failingTeam  + "," + proposes + ")";
        String ruleSupportSuccess = "<" + supportsSuccessfulTeam + " ==> (--,<$1 --> spy>)>. %0.60;0.70%\n";
        String ruleAgainstSuccess = "<" + againstSuccessfulTeam + " ==> <$1 --> spy>>. %0.60;0.70%\n";
        String ruleSupportFail = "<" + supportsUnsuccessfulTeam + " ==> <$1 --> spy>>. %0.70;0.70%\n";
        String ruleAgainstFail = "<" + againstUnsuccessfulTeam + " ==> (--,<$1 --> spy>)>. %0.70;0.70%\n";
        String ruleProposeSuccess = "<" + proposesSuccessfulTeam + " ==> (--,<$1 --> spy>)>. %0.60;0.70%\n";
        String ruleProposeFail = "<" + proposesUnsuccesfulTeam + " ==> (--,<$1 --> spy>)>. %0.60;0.70%\n"; //May need to adjust probabilities

        //Rules about vouching/accusing
        String twoPlayers = "<$3 --> player>,<#4 --> player>";
        String accuses = "<(*, $3, <#4 --> spy>) --> believes>";
        String vouches = "<(*, $3, (--, <#4 --> spy>)) --> believes>";
        String oneIsSpy = "<$3 --> spy>";

        String ruleSpyAccuse = "<(&&," + twoPlayers + "," + oneIsSpy + "," + accuses + ") ==> (--,<#4 --> spy>)>. %0.60;0.60%\n";
        String ruleSpyVouches = "<(&&," + twoPlayers + "," + oneIsSpy + "," + vouches + ") ==> <#4 --> spy>>. %0.60;0.6%\n";


        return ruleMissionSuccess + "10\n" + ruleMissionFail + "10\n" + ruleSupportSuccess + "10\n" + ruleAgainstSuccess +
                "10\n" + ruleSupportFail + "10\n" + ruleAgainstFail + "10\n" + ruleProposeSuccess + "10\n" +
                ruleProposeFail + "10\n" + ruleSpyAccuse + "10\n" + ruleSpyVouches + "10\n";

    }

    public static String getQuestions(){
        if(spiesKnown)
            return "";

        //Ask a question about each player if they are a spy
        String narsese = "";
        for(int i = 1; i <= numPlayers; i++)
        {
            String player = Player.playerNames.get(i) + gameNumber;
            //narsese += "(^wonder, {SELF},<{" + player + "} --> spy>)!\n";
            narsese += "<{" + player + "} --> spy>?\n";
            narsese += "''outputMustContain('<{" + player + "} --> spy>?' )\n";
            narsese += "300\n";
        }
        return narsese;

    }
}


















